#include "SuperFAST.h"

#include <opencv2/opencv.hpp>
#include <opencv2/features2d/features2d.hpp>

#include <vector>

// integer constants
enum
{
    FAST9_FIXED_THRESHOLD = 25
};

int main(int argc, char *argv[])
{
    if (argc != 2)
    {
        std::cerr << "usage: " << argv[0] << " <VGA-video-path>\n";
        return 1;
    }

    cv::VideoCapture cap(argv[1]);

    if (!cap.isOpened())
    {
        std::cerr << "error: couldn't open video!\n";
        return 1;
    }

    int gridDimensions[SUPERFAST_LEVEL_PYRAMID_SIZE][2] = {
      { 6, 4 }, // level 0 - VGA
      { 3, 2 }, // level 1 - QVGA
      { 1, 1 }, // level 2 - QQVGA
      { 1, 1 }  // level 3 - "QQQVGA"
    };
    // int gridDims[SUPERFAST_LEVEL_PYRAMID_SIZE][2] = {
    //     { 1, 1 }, { 1, 1 }, { 1, 1 }, { 1, 1 }
    // };
    int desiredCornerCounts[SUPERFAST_LEVEL_PYRAMID_SIZE] = {
      2000,
      1300,
      600,
      500
    };

    SuperFAST superFAST;

    superFAST.init(gridDimensions, desiredCornerCounts);

    cv::Mat frame;
    cv::Mat grayImg;

    for (;;)
    {
        if (!cap.read(frame))
            break;

        cv::cvtColor(frame, grayImg, CV_BGR2GRAY);

        SuperFASTImageHolder vgaImg(
            grayImg.data, grayImg.size().width, grayImg.size().height);
        SuperFASTExtractionResults extractionResults;

        superFAST.extractCorners(vgaImg, &extractionResults);

        std::cout << "SuperFast corner count: " << extractionResults.corners[0]
                                                       .cornerCount << "\n";

        // cv::Mat qqqvgaLevel(extractionResults.image[3].getHeight(),
        //                     extractionResults.image[3].getWidth(),
        //                     CV_8UC1,
        //                     extractionResults.image[3].getData());

        std::vector<cv::KeyPoint> keypoints;
        cv::FAST(grayImg,
                 keypoints,
                 FAST9_FIXED_THRESHOLD,
                 /*nonmaxSuppression=*/false);

        std::cout << "Fast-9 corner count: " << keypoints.size() << "\n";
    }

    return 0;
}
