#ifndef SUPERFAST_H
#define SUPERFAST_H

#if __GNUC__ >= 4
#define SUPERFAST_API __attribute__((visibility("default")))
#else
#define SUPERFAST_API
#endif

class SuperFASTImpl;

#define SUPERFAST_LEVEL_PYRAMID_SIZE 4

/// \brief A light object, that wraps a single channel image represented
/// contiguously in memory.
///
/// \note This object doesn't own the data and becomes invalid when the
/// underlying data is released.
class SuperFASTImageHolder
{
public:
    SuperFASTImageHolder() : data_(0), width_(0), height_(0)
    {
    }

    SuperFASTImageHolder(unsigned char *data, int width, int height)
      : data_(data), width_(width), height_(height)
    {
    }

    inline unsigned char *getData() const
    {
        return data_;
    }

    inline int getWidth() const
    {
        return width_;
    }

    inline int getHeight() const
    {
        return height_;
    }

private:
    unsigned char *data_;
    int width_;
    int height_;
};

struct SuperFASTCorner
{
    int x;
    int y;
};

struct SuperFASTCorners
{
    unsigned cornerCount;
    SuperFASTCorner *corners;
};

struct SuperFASTExtractionResults
{
    SuperFASTImageHolder image[SUPERFAST_LEVEL_PYRAMID_SIZE];
    SuperFASTCorners corners[SUPERFAST_LEVEL_PYRAMID_SIZE];
};

class SUPERFAST_API SuperFAST
{
public:
    SuperFAST();
    ~SuperFAST();

    void init(const int gridDimensions[SUPERFAST_LEVEL_PYRAMID_SIZE][2],
              const int desiredPointCounts[SUPERFAST_LEVEL_PYRAMID_SIZE]);

    /// \param[out] results
    void extractCorners(SuperFASTImageHolder imageHolder,
                        SuperFASTExtractionResults *results);

private:
    SuperFASTImpl *const pimpl;
};

#endif // SUPERFAST_H
